package com.glavsoft.drawing;

//import java.io.UnsupportedEncodingException;

public class RC5_48{
    public int nr;
    public int [] xk;

    public RC5_48(int rounds)
    {
        nr = rounds;
        xk = new int[4*(rounds*2+2)];
    }

    public int ROTL24(int x, int c){
        int rot = ((x<<c) & 0x00ffffff) | ((x>>>(24-c)) & 0x00ffffff);
        //int rot = (x<<c) | (x>>>(24-c));
        return rot;
    }

    public int ROTR24(int x, int c){
        int rot = ((x>>>c) & 0x00ffffff) | ((x<<(24-c)) & 0x00ffffff);
        //int rot = (x>>>c) | (x<<(24-c));
        return rot;
    }


public void rc5_destroy()
{
    for(int i=0; i<nr*2+2; ++i)
        xk[i] = 0;
}


public int [] rc5_encrypt(int [] data, int blocks, int x)
{
    int [] d, sk;
    int h, i, rc;

    d = new int[2];
    sk = xk;

    for(h=0; h<blocks; ++h){
        d[0] = data[h*2] & 0x00ffffff;
        d[1] = data[h*2+1] & 0x00ffffff;
        d[0] += (xk[0] ^ (((x+nr)*nr)&0x00ffffff)) & 0x00ffffff;
        d[0] &= 0x00ffffff;
        d[1] += (xk[1] ^ (((x+nr)*nr)&0x00ffffff)) & 0x00ffffff;
        d[1] &= 0x00ffffff;
        for(i=0; i<nr*2; i+=2){
            d[0] ^= d[1] & 0x00ffffff;
            rc = ((d[1] & 23) & 0x00000017);
            d[0] = ROTL24(d[0], rc) & 0x00ffffff;
            d[0] &= 0x00ffffff;
            d[0] += sk[i+2] & 0x00ffffff;
            d[0] &= 0x00ffffff;
            d[1] ^= d[0] & 0x00ffffff;
            rc = ((d[0] & 23) & 0x00000017);
            d[1] = ROTL24(d[1], rc) & 0x00ffffff;
            d[1] &= 0x00ffffff;
            d[1] += sk[i+3] & 0x00ffffff;
            d[1] &= 0x00ffffff;
        }
        data[h*2] = d[0] & 0x00ffffff;
        data[h*2+1] = d[1] & 0x00ffffff;
    }
    return data;
}

public int [] rc5_decrypt(int [] data, int blocks, int x)
{
    int [] d, sk;
    int h, i, rc;

    d = new int[2];
    sk = xk;

    for(h=0; h<blocks; ++h){
        d[0] = data[h*2] & 0x00ffffff;
        d[1] = data[h*2+1] & 0x00ffffff;
        for(i=nr*2-2; i>=0; i-=2){
            d[1] -= sk[i+3] & 0x00ffffff;
            d[1] &= 0x00ffffff;
            rc = ((d[0] & 23) & 0x00000017);
            d[1] = ROTR24(d[1], rc) & 0x00ffffff;
            d[1] &= 0x00ffffff;
            d[1] ^= d[0] & 0x00ffffff;
            d[0] -= sk[i+2] & 0x00ffffff;    
            d[0] &= 0x00ffffff;
            rc = ((d[1] & 23) & 0x00000017);
            d[0] = ROTR24(d[0], rc) & 0x00ffffff;
            d[0] &= 0x00ffffff;
            d[0] ^= d[1] & 0x00ffffff;
        }
        d[0] -= (xk[0] ^ (((x+nr)*nr)&0x00ffffff)) & 0x00ffffff;
        d[0] &= 0x00ffffff;
        d[1] -= (xk[1] ^ (((x+nr)*nr)&0x00ffffff)) & 0x00ffffff;
        d[1] &= 0x00ffffff;
        
        data[h*2] = d[0] & 0x00ffffff;
        data[h*2+1] = d[1] & 0x00ffffff;
    }
    return data;
}


public void rc5_key(byte [] key, int keylen, int y)
{
    int [] pk;
    int A, B;
    int xk_len, pk_len, i, j, k, num_steps, rc;

    xk_len = nr*2 + 2;
    pk_len = keylen / 4;
    if((keylen%4) != 0)
        pk_len += 1;

    pk = new int[pk_len];

    for(i=0,j=0; i<pk_len; ++i,j+=4){
    	pk[i]=0;
    	for(k=0; k<4; ++k){
    		pk[i] <<= 8;
    		if((j+k) < keylen){
    			pk[i] |= key[j+k] & 0x0ff;
    			pk[i] ^= (y+keylen)*keylen;
    		}
    	}
    }

    xk[0] = 0x00b7e151;
    for(i=1; i<xk_len; ++i){
        xk[i] = (xk[i-1] + 0x009e3779) & 0x00ffffff;
    }
    
    if(pk_len > xk_len)
        num_steps = 3 * pk_len;
    else
        num_steps = 3 * xk_len;

    A = B = 0;
    for(i=0; i<num_steps; ++i){
        A = xk[i % xk_len] = ROTL24(xk[i % xk_len] + A + B, 3);
        rc = (((A+B) & 23) & 0x00000017);
        B = pk[i % pk_len] = ROTL24((pk[i % pk_len] + A + B) & 0x00ffffff, rc) & 0x00ffffff;
    }

    for(i = 0; i<pk_len; ++i)
        pk[i] = 0;
}

/*
public static void main(String [] argv)
{
    int i, keylen, words = 30;
    int [] data = new int[words];
    byte [] key;
    
    String str = "08237061";
    
    RC5_48 c = new RC5_48(20);
    
    try{
        key = str.getBytes("US-ASCII");
        keylen = str.length();
        for(i=0; i<keylen; ++i)
            System.out.printf("%c %04x\n", key[i], key[i]);
        c.rc5_key(key, keylen);
    } catch (UnsupportedEncodingException e) {  
        e.printStackTrace();  
    }  
    
    System.out.printf("---------------------------------------------\n");
    for(i=0; i<words; ++i){
        data[i] = i;
    }



    data = c.rc5_encrypt(data, words/2);
    System.out.printf("Encryptions:\n");
    for(i=0; i<words; i+=2){
        System.out.printf("Block %01d = %08x %08x\n",
                i/2, data[i] & 0x00ffffff, data[i+1] & 0x00ffffff);
    }
   
    data = c.rc5_decrypt(data, words/2);
    System.out.printf("Decryptions:\n");
    for(i=0; i<words; i+=2){
        System.out.printf("Block %01d = %08x %08x\n",
                i/2, data[i], data[i+1]);
    }

    c.rc5_destroy();
}
*/
}
