/*
 * hvm.c : FBCrypt sends input to VMM
 * 
 */

#include "./fbcrypt.h"
#include "../hw/ps2.h"

#define PS2_QUEUE_SIZE 256


typedef struct {
	#if FBCRYPT_HVM
    uint32_t data[PS2_QUEUE_SIZE];
	#else
    uint8_t data[PS2_QUEUE_SIZE];
	#endif
    int rptr, wptr, count;
} PS2Queue;

typedef struct {
    PS2Queue queue;
    int32_t write_cmd;
    void (*update_irq)(void *, int);
    void *update_arg;
} PS2State;

typedef struct {
    PS2State common;
    int scan_enabled;
    /* Qemu uses translated PC scancodes internally.  To avoid multiple
       conversions we do the translation (if any) in the PS/2 emulation
       not the keyboard controller.  */
    int translate;
    int scancode_set; /* 1=XT, 2=AT, 3=PS/2 */
} PS2KbdState;

typedef struct {
    PS2State common;
    uint8_t mouse_status;
    uint8_t mouse_resolution;
    uint8_t mouse_sample_rate;
    uint8_t mouse_wrap;
    uint8_t mouse_type; /* 0 = PS2, 3 = IMPS/2, 4 = IMEX */
    uint8_t mouse_detect_state;
    int mouse_dx; /* current values, needed for 'poll' mode */
    int mouse_dy;
    int mouse_dz;
    uint8_t mouse_buttons;
} PS2MouseState;


int HVM_keyEvent2VMM(int down, uint32_t sym)
{
	PS2KbdState *ps2kbd = kbd_put_keycode2();

	/* Not convert a sym(encrypted) to the keycode.
	 * direct write the sym in the queue. */
	ps2_queue( (&ps2kbd->common), sym);
	return 0;
}


int SHA_HVM_keyEvent2VMM(int down, uint32_t sym, FBCRYPT_info *fbc)
{
	PS2KbdState *ps2kbd = kbd_put_keycode2();

	/* HYPER_CALL No.41 send MAC */
	hypercall_v2(__HYPERVISOR_hvm_mac,1,(void *)fbc);

	/* Not convert a sym(encrypted) to the keycode.
	 * direct write the sym in the queue. */
	ps2_queue( (&ps2kbd->common), sym);
	return 0;
}

