/*
 *  sha-fbcrypt.c -- Get SHA1_hash and check integrity for FBCrypt
 *
 */

#include <xen/lib.h>
#include <xen/string.h>
#include <crypto/fbcrypt/cyassl/sha1.h>
#include <crypto/fbcrypt/configure.h>

/**
 * itoa: converts an integer to a string
 *
 * assumes the string passed to it is of
 * sufficient length, returns the length
 * of the string.
 */

static int itoa(char s[], int n, int i)
{
    if (n < 0) {
        s[i++] = '-';
        n = -n;
    }

	if (n / 10) {
        i = itoa(s, n / 10, i);
        s[i++] = n % 10 + '0';
    }
	else {
        s[i++] = n % 10 + '0';
        s[i] = '\0';
    }

	return i;
}

static int chkSHA(unsigned int hash[], unsigned int send[])
{
	#if DEBUG_VKBD_INFO
	int i = 0;
	printk("calc:");
	for( i = 0; i < 5; i++ ) { printk("%08x ",hash[i]); } 
	printk("\n");
	printk("sent:");
	for( i = 0; i < 5 ; i++ ) { printk("%08x ",send[i]); } 
	printk("\n");
	#endif

	if ( !(hash[0]==send[0] && hash[1]==send[1] && 
		 hash[2]==send[2] && hash[3]==send[3] && hash[4]==send[4]) ) {
		return 1; /* not matched */
	}

	return 0; /* matched */
}

int getSHA2(int key, int seq, unsigned int send_hash[])
{
    SHA1Context sha;
	char *value = NULL;
    char tmp[10] = {'\0'};

	/* int -> char */
    itoa(tmp, key+seq, 0);
	value = tmp;

    SHA1Reset(&sha);
    SHA1Input(&sha, (const unsigned char*)value, strlen(value));

    if (!SHA1Result(&sha)) 
		printk("ERROR-- could not compute message digest\n"); 
    else if ( chkSHA(sha.Message_Digest,send_hash) ) {
		printk("xxxxxxxxx SHA1 NOT matched xxxxxxxxxxx\n");
		printk("input:%d seq:%d\n",key,seq);
		return 1;
	}
	#if DEBUG_VKBD_INFO
	printk("========= SHA1 matched ==========\n");
	printk("input:%d seq:%d\n",key,seq);
	#endif

    return 0;
}
