/*--------------------------------------------------
 * xen/fbcrypt/vkbd_decrypt.c
 *
 * QEMU から呼ばれるハイパーコール
 * VMM からvkbdのI/Oリングにキーボード入力を書き込む
 *
 * ------------------------------------------------*/
 
#include <asm/system.h>
#include <xen/domain_page.h>
#include <xen/lib.h>
#include <xen/types.h>
#include <public/io/kbdif.h>
#include <xen/time.h>

#include <crypto/fbcrypt/keycode/keycode.h>
#include <crypto/fbcrypt/pv/pv.h>
#include <crypto/fbcrypt/cyassl/cyassl.h>

static VncState vs;
static int init_flag=0;

static unsigned int vkbd_decrypt(int domid, unsigned int encrypt, int reset_key)
{
	dominfo_p p = Domain_get_pointer(domid);

	if ( reset_key ) {
		RC4_reset(domid);
	}

	if ( p == NULL ) {
		printk("ERROR: vkbd_decrypt p==NULL\n");
		return 1;
	}

	return AES_kbd_decrypt(p,encrypt,reset_key);
	//return RC4_key_decrypt(p,encrypt);
}


/*---------------------------------  HYPERCALL No.39
 * QEMUの仮想キーボードドライバをVMMに移植
 * ioemu-qemu-xen/vnc.c/keyEvent2VMM/HC_vkbd_write から実行

 * reference QEMU hw/xenfb.c
 *------------------------------------------------*/
asmlinkage int do_vkbd_write(union xenkbd_in_event *event, FBCRYPT_info *fbc)
{
	int domid 	= fbc->domid;
	int mode  	= fbc->mode;
	int sha1chk = fbc->sha;
	int seq  	= fbc->seq;
	int reset  	= fbc->reset;
	#if TIME_PV_KBD
    s_time_t start;
	s_time_t end;
	/* don't use double. VMM can't deal it */
	s_time_t overhead;
	static unsigned long logid = 0;
	#endif

	/* Get IO-ring  */
	struct xenkbd_page *page = Domain_mfn2map_page(domid,VMM_VKBD);
	uint32_t prod = page->in_prod;
	int down = event->key.pressed;
	unsigned int decrypt = 0;
	
	#if TIME_PV_KBD
   	start = NOW();
	#endif

	if ( init_flag == 0 ) {
		init_vs2(&vs);
		init_flag = 1;
		printk("init_vs OK\n");
	}

	/* check valid I/O-ring */
	if ( prod - page->in_cons == XENKBD_IN_RING_LEN ) {
		unmap_domain_page(page);
		return -1;
	}

	/* only keyboard */
	if ( event->type == 3 ) {
		#if DEBUG_VKBD_INFO
		printk("\n--------------------------------------\n");
		printk(" HYPERCALL: vkbd I/O RING Write INFO\n");
		printk("--------------------------------------\n");
		printk("Domain_ID:%d mode PV:%d\n",domid,mode);
		printk("seq:%d reset:%d sha1Flag:%d\n",seq,reset,sha1chk);
		#endif

		if ( mode == FBCRYPT_RC4 )  {

			/* Check SHA1 */
			if ( sha1chk ) {
				if ( getSHA2(event->key.keycode,seq,fbc->hash) ) {
					/* DO NOT WRITE IO-Ring */
					//return 1;
					printk("sha1 not matched\n");
				}
			}

			/* AES decrypt */
			decrypt = vkbd_decrypt(domid,event->key.keycode,reset);

			/* convert decrypt -> keycode */
			event->key.keycode = do_key_event(&vs,down,decrypt,mode);
		} 
		else if ( mode == NO_ENCRYPT )  {
			event->key.keycode = do_key_event(&vs,down,event->key.keycode,0);
		}

		#if DEBUG_VKBD_INFO
		printk("decrypt: %c(%d) -> keycode:%d pressed:%d\n",
				(unsigned char)decrypt,decrypt,event->key.keycode,event->key.pressed);
		printk("---------------------------------------\n");
		#endif
	}

	/* write data on IO-ring */
    mb();		/* ensure ring space available */
    XENKBD_IN_RING_REF(page, prod) = *event;
    wmb();		/* ensure ring contents visible */
    page->in_prod = prod + 1;

	unmap_domain_page(page);

	#if TIME_PV_KBD
    end = NOW();
	overhead = (end-start);
    printk("ID:%ld PV_INPUT_DECRYPT: %ld(ns)\n",logid++,overhead);
	#endif

	return 0;
}

