/*
 * FBCrypt common_functions
 * 
 */

#include <sys/types.h>
#include <xs.h>
#include <xenctrl.h>
#include <xen/io/fbif.h>
#include <xen/io/kbdif.h>

#define FBCRYPT_COMMON 1	/* ON:1 OFF:0 FBCRYPT_COMMON */
#define FBCRYPT_PV 	0		/* ON:1 OFF:0 FBCRYPT_PV */
#define FBCRYPT_HVM 1		/* ON:1 OFF:0 FBCRYPT_HVM */

#define MYKEYLOGGER 0  		/* OFF:0 ON:1 Keylogger */
#define TIME_VKBD_HC 1		/* OFF:0 ON:1 Time */
#define TIME_SERVER_OVERHEAD_KBD 0	/* OFF:0 ON:1 Time */
#define TIME_SERVER_OVERHEAD_SCREEN 1	/* OFF:0 ON:1 Time */

#define TIME_ABSOLUTE 1 /* new */

#define TIME_SERVER_OVERHEAD_PV 1	/* OFF:0 ON:1 Time */
#define TIME_SERVER_OVERHEAD_HVM 0	/* OFF:0 ON:1 Time */

#define COMMON_DEBUG_KBD 0	/* OFF:0 ON:1 DEBUG_INFO for HVM*/
#define HVM_DEBUG_KBD 0		/* OFF:0 ON:1 DEBUG_INFO for HVM*/
#define PV_DEBUG_KBD  0		/* OFF:0 ON:1 DEBUG_INFO for PV*/
#define DEBUG_LAYOUT 0
#define DEBUG_HVM_SC 0		/* DEBUG HVM SCREEN */

#define MYLOG() printf("%s:%d\n",__FUNCTION__,__LINE__)

/* SERVER_OVERHEAD_KBD */
extern double fbcrypt_time_kbd_start;
extern double fbcrypt_time_kbd_end;
extern int fbcrypt_time_kbd_flag;
extern double fbcrypt_kbd_overhead;

/* SERVER_OVERHEAD_SCREEN */
extern double fbcrypt_time_fb_start;
extern double fbcrypt_time_fb_end;
extern int fbcrypt_time_fb_flag;
extern double fbcrypt_screen_overhead;

typedef enum virtualization_type {
	HVM = 1,
	PV  = 2,
} V_TYPE;

typedef struct FBCRYPT_info {
	V_TYPE mode;
	int domid;
	int down;
	int seq;
	int reset;
	int sha;
	int hash[4];
} FBCRYPT_info;

typedef struct kbd_layout_t *kbdLayout;

/* TIME_LOG */
int my_timelog(double time);
int my_timelog2(double time);
int HC_get_time_now(int stop); /* stop:1 start:1 */
double my_gettimeofday_sec(void);

/* keylogger */
int vnc_keylogger2(unsigned int buf);

/* ioemu-qemu-xen vl.c --------------------------------------*/
void *kbd_put_keycode2(void);	/* get opaque */

/* COMMON */
int keyEvent2VMM(int down, int seq, uint32_t sym);
int SHA_keyEvent2VMM(int down, int seq, uint32_t sym, 
		uint32_t hash0, uint32_t hash1, uint32_t hash2, uint32_t hash3, uint32_t hash4);

/* HVM */
int HVM_keyEvent2VMM(int down, uint32_t sym);
int SHA_HVM_keyEvent2VMM(int down, uint32_t sym, FBCRYPT_info *fbc);

/* PV */
int PV_keyEvent2VMM(uint32_t sym, FBCRYPT_info *fbc);

/* HYPERCALL */
int hypercall_v2(int op, int cnt, ...);

/* for DEBUG */
void HC_CHECK_PRINT(unsigned char *s);
