/*
 * pv.c : FBCrypt sends input to VMM
 * 
 */

/* xenkbd --------------------------- */
#include <xenctrl.h>
#include <xen/event_channel.h>
#include <xen/io/xenbus.h>
#include <xen/io/fbif.h>
#include <xen/io/kbdif.h>
#include <xen/io/protocols.h>
#include "../qemu-common.h"
#include "../console.h"
#include "../sysemu.h"
#include "../qemu_socket.h"
#include "../qemu-timer.h"

/* hw/xenfb ------------------------- */
#include <stdarg.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdbool.h>
#include <sys/mman.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include "../hw/xen_backend.h"
/*------------------------------------ */
#include "fbcrypt.h"


/* hw/xenfb.c ----------------------------------------- */
struct common {
    struct XenDevice  xendev;  /* must be first */
    void              *page;
    DisplayState      *ds;
};

struct XenInput {
    struct common c;
    int abs_pointer_wanted; /* Whether guest supports absolute pointer */
    int button_state;       /* Last seen pointer button state */
    int extended;
    QEMUPutMouseEntry *qmouse;
};


int PV_keyEvent2VMM(uint32_t sym, FBCRYPT_info *fbc)
{
    union xenkbd_in_event event;
	struct XenInput *xenfb = NULL;
	int ret;

	#if MYKEYLOGGER
	if (fbc->down) vnc_keylogger2(sym);
	#endif

	/* get guest domain id */
	xenfb = kbd_put_keycode2();
	if ( xenfb == NULL ) printf("xenfb == NULL\n");
	fbc->domid = xenfb->c.xendev.dom;
	
	/* set event (hw/xenfb.c/xenfb_send_key) */
	memset(&event, 0, XENKBD_IN_EVENT_SIZE);
	event.type = XENKBD_TYPE_KEY;
	event.key.pressed = fbc->down;
	event.key.keycode = sym;

	#if PV_DEBUG_KBD
	printf("domid:%d\n",fbc->domid);
	#endif
	
	/* HYPER_CALL No.39 write encrypted data to the I/O Ring */
	ret = hypercall_v2(__HYPERVISOR_vkbd_write,2,(void *)&event,(void *)fbc);
	if (ret) printf("SHA1 CHECK Don't Match\n");
	
	/* send event (hw/xen_backend.c) */
	return xen_be_send_notify(&xenfb->c.xendev);
}

